geo.coordinates <- function(x, output="Output", unit="dec", type=F, short=F, generic=F, class=F, elev=F, note=F)
  {

if(missing(x)) # Se nao fornece arquivo de entrada
 {stop("Você deve fornecer um arquivo de localidades para a função (argumento x).\nConsulte o help para saber como deve ser o arquivo.")}

# Caso o usuario digite errado a unidade de coordenada desejada:
if(!(unit %in% c("dms", "dec", "mgrs"))) # if(unit==!"dms"|unit==!"dec"|unit==!"mgrs") - opcao
{stop("É necessário escolher umas das opções de unidade de coordenadas no argumento unit: dms, dec ou mgrs. Confira a grafia")}

###################################################################
############### Importando e organizando database #################

if(!(any("zip"==dir(getwd())))) # Para nao criar pasta zip de novo caso ja exista
  {dir.create("zip")}

work.dir <- getwd() # Salvar o diretorio de trabalho
setwd("zip") # Mandar para pasta criada para a descompressao de arquivos

if(!(any("br.zip"==dir(getwd())))) # Para nao perder tempo baixando caso ja o tenha feito
  {download.file("http://earth-info.nga.mil/gns/html/cntyfile/br.zip", "br.zip")
  unzip("br.zip")}

database <- read.delim("br.txt", fileEncoding="utf8") # Lendo Banco de Dados

setwd(work.dir) # Voltando ao diretorio de trabalho

database <- database[,c(4,5,6,7,8,10,16,18,20,21,23,28)] # Apenas colunas de interesse
colnames(database) <- c("lat_dec", "long_dec", "lat_dms", "long_dms", "coord_mgrs", "class", "elev", "type", "short", "generic", "full", "note") # Reduzindo nome das colunas

### Trocando tudo para letras minusculas nas colunas full e generic
for(m in 10:11)
  {
  database[,m] <- tolower(database[,m])
  }

### Substituindo acentuacao nas colunas full e generic
letras.1 <- c("á", "à", "ã", "â", "ä", "é", "è", "ê", "í", "ó", "ò", "õ", "ô", "ú", "ü", "ñ", "ç")
letras.2 <- c("a", "a", "a", "a", "a", "e", "e", "e", "i", "o", "o", "o", "o", "u", "u", "n", "c")

for(l in 1:length(letras.1))
  {
  for(v in 10:11)
    {
    tempo <- grep(letras.1[l], database[, v])
    
    if(length(tempo)!=0)
      {database[tempo, v] <- gsub(letras.1[l], letras.2[l], database[tempo, v])}
    }
  }

############## Fim do Importando e organizando database ##############
######################################################################
######################################################################
################## Importando e organizando dados ####################

if(length(x)==1) # Se da o caminho (um elemento)
{dados.original <- scan(x, "character", sep="\n", skip=1)}

else # Se ja da vetor (mais de um elemento)
{dados.original <- x}

## Minusculas
dados <- tolower(dados.original) # Tambem em minusculas para evitar problemas na comparacao de caracteres

## Sem acentos
for(l in 1:length(letras.1))
  {
  dados[grep(letras.1[l], dados)] <- gsub(letras.1[l], letras.2[l], dados[grep(letras.1[l], dados)])
  }

############## Fim do Importando e organizando dados ###############
####################################################################
####################################################################
################ Importando shapefiles para mapa ###################

setwd("zip") # Direcionando para a pasta de descompressao

if(!(any("BRA_adm.zip"==dir(getwd())))) # So baixar o mapa se o shapefile ainda nao existir
  {download.file("http://gadm.org/data/shp/BRA_adm.zip", "BRA_adm.zip")
  unzip("BRA_adm.zip")}

if(unit=="dec") # Abrindo pacotes para criacao de mapa (para unidade decimal de coordenadas)
  {require("sp")
  require("maptools")
  brazil.shape <- readShapePoly("BRA_adm1",
                                proj4string=CRS(as.character(NA)))}

setwd(work.dir) # Voltar ao diretorio de trabalho

########### Fim do Importando shapefile para mapa ############
##############################################################
##############################################################
#################### Definindo argumentos ####################

colunas <- c(11) # Coluna minima que constara no output: o nome das localidades

# A escolha dos argumentos determina quais colunas serao adicionadas no output da funcao:

if(unit=="dec")
{colunas <- c(colunas,1,2)}
if(unit=="dms")
  {colunas <- c(colunas,3,4)}
if(unit=="mgrs")
  {colunas <- c(colunas,5)}

if(type==T)
  {colunas <- c(colunas, 8)}

if(short==T)
  {colunas <- c(colunas, 9)}

if(generic==T)
  {colunas <- c(colunas, 10)}

if(class==T)
  {colunas <- c(colunas, 6)}

if(elev==T)
  {colunas <- c(colunas, 7)}

if(note==T)
  {colunas <- c(colunas, 12)}

############## Fim do Definindo argumentos ###############
##########################################################
##########################################################
############## Buscando dados no database ################

nlines <- length(dados) # Numero de observacoes/localidades do usuario
parcial <- list() # Lista vazia onde colocar o resultado da busca

## Cruzando tabelas e salvando resultados numa lista com listas
for(b in 1:nlines)
  {
  parcial[[b]] <- database[grep(dados[b], database$full), colunas]
  }

## Opcoes de resultados encontrados
locais.zero <- integer() # Criando onde guardar os indices de quem nao foi encontrado
result <- data.frame(full=character(), lat=numeric(), long=numeric()) # Criando data frame onde jogar os resultados
nome.original <- character() # Para criar coluna com nomes originais no data.frame final

for(i in 1:length(parcial)) # Percorrendo a tabela de resultados
  {
  if(dim(parcial[[i]])[1]==0) # Se encontra uma localidade sem correspondente no database
    {
    locais.zero <- c(locais.zero, i) # Salva o indice da localidade no objeto
    }

  if(dim(parcial[[i]])[1]==1) # Se encontra exatamente um registro
    {
    result <- rbind(result, parcial[[i]]) # Joga o registro num data frame que constituira o output final
    nome.original <- c(nome.original, dados.original[i]) # Para coluna de nomes originais
    }

  if(dim(parcial[[i]])[1]>1) # Se encontra mais de uma coordenada para uma mesma localidade
    {
    show <- data.frame(indice=1:dim(parcial[[i]])[1], parcial[[i]]) # data.frame de possiveis localidades
    cat("\n")
    write.table(show, quote=F, row.names=FALSE) # Mostra as opcoes
    
    if(unit=="dec")
    {plot(brazil.shape) # Plota mapa
    text(show[,4:3], labels=as.character(show$indice), col=2)} # Plotando as possibilidades para ajudar na escolha
    
    temp <- readline(prompt="\nHá mais de uma opção para essa localidade (tabela acima).\nO mapa pode ajudar na identificação (só para unidade decimal de coordenadas).\nEscolha o registro correto e digite o índice correspondente\n(digite zero para nenhuma das opções):") # Pede para escolher a opcao que quer
    while(!(any(temp==(0:dim(parcial[[i]])[1])))) # Se digitar algo que nao uma das opcoes, pede de novo!
      {
      cat("\nEscolha uma das opções fornecidas, indicadas pelo índice numérico:\n\n")
      write.table(show, quote=F, row.names=FALSE)
      temp <- readline(prompt="\nHá mais de uma opção para essa localidade (tabela acima).\nO mapa pode ajudar na identificação (só para unidade decimal de coordenadas).\nEscolha o registro correto e digite o índice correspondente\n(digite zero para nenhuma das opções):")
      }
    
    if(temp==0) # Se encontra uma localidade sem correspondente no database
      {
      locais.zero <- c(locais.zero, i) # Salva o indice da localidade no objeto
      }
    else
    {result <- rbind(result, show[show$indice==temp, c(-1)]) # Salva a escolha no data.frame final
    nome.original <- c(nome.original, dados.original[i])}
    }
  }

rownames(result) <- 1:dim(result)[1] # Enumerando as linhas do output
result$original <- nome.original # Criando coluna com nomes originais
result <- result[c(length(result), 1:(length(result)-1))] # Jogando a coluna de nomes originais para o comeco da tabela

############## Fim do Buscando dados no database ###############
################################################################
################################################################
############## Salvando e reportando os resultados #############

write.table(x=result, file=paste(output, "txt", sep="."), quote=F, sep="\t") # Salvando o resultado num arquivo na pasta de trabalho

if(unit=="dec") # Salvando o mapa resultante na pasta de trabalho
  {pdf(paste(output, "Map.pdf", sep="_"))
  map <- plot(brazil.shape) # Plotando mapa
  text(result[,4:3], labels=rownames(result), col=2) # Plotando dados
  dev.off()}

if(length(locais.zero)>0) # Avisando quais localidades nao foram encontradas
  {cat("\nAs seguintes localidades não foram encontradas no GEONet Names Server: ", paste(dados.original[locais.zero],collapse=", "), "\nVocê pode tentar checar a grafia dos nomes.\n")}

# Avisando o que aconteceu com os resultados
cat("\nA tabela com as coordenadas foi salva em: ", paste(output, "txt", sep="."), "\nSe você utilizou decimal como unidade das coordenadas geográficas, um mapa com as localidades encontradas foi salvo em: ", paste(output, "Map.pdf", sep="_"), "\n\n")
  
return(result)

  } # Fim da Funcao geo.coordinates()! Ufa!
